-- Copyright (c) 2011 Cloudera, Inc. All rights reserved.
--
-- Version 1 of the CMON schema. The base schema; modifications are made via
-- separate pieces of migration code.
--

-- The schema version, should be checked by the server during startup
create table CMON_SCHEMA_VERSION (
    VERSION integer not null
) engine=InnoDB;

-- Update this value when the schema changes, also during migration
insert into CMON_SCHEMA_VERSION set VERSION=1;

create table CMON_METRIC_INFO (
  METRIC_ID    smallint unsigned not null,
  NAME         varchar(255) not null,
  primary key (METRIC_ID)
) ENGINE=InnoDB;

create table CMON_LONG_LIVED_TIME_SERIES_ID (
  ID         mediumint unsigned not null auto_increment,
  CATEGORY   bigint not null,
  NAME       varchar(255) not null,
  primary key (ID),
  index IDX_SERIES_BY_NAME (NAME)
) ENGINE=InnoDB;

create table CMON_LONG_LIVED_TIME_SERIES_ATTRIBUTE (
  ID         bigint not null auto_increment,
  ATTR_KEY   varchar(255) not null,
  VALUE      varchar(255) not null,
  SERIES_ID  mediumint unsigned not null,
  primary key (ID),
  index IDX_LLATTR_ID (SERIES_ID),
  constraint FK_LLATTR_SERIES_ID foreign key (SERIES_ID) references CMON_LONG_LIVED_TIME_SERIES_ID (ID)
) ENGINE=InnoDB;

create table CMON_LONG_LIVED_DATA_POINTS_HOURLY (
  ID          bigint not null auto_increment,
  SERIES_ID   mediumint unsigned not null,
  METRIC_ID   smallint unsigned not null,
  START_TS    bigint not null,
  TS          bigint not null,
  END_TS      bigint not null,
  AVERAGE     double not null,
  MINIMUM     double not null,
  MAXIMUM     double not null,
  primary key (ID),
  index IDX_SERIES_METRIC_TS (SERIES_ID, METRIC_ID, TS),
  constraint FK_LLHOURLY_SERIES_ID foreign key (SERIES_ID) references CMON_LONG_LIVED_TIME_SERIES_ID (ID)
) ENGINE=InnoDB;

create table CMON_LONG_LIVED_DATA_POINTS (
  ID          bigint not null auto_increment,
  SERIES_ID   mediumint unsigned not null,
  METRIC_ID   smallint unsigned not null,
  TS          bigint not null,
  VALUE       double not null,
  primary key (ID),
  index IDX_SERIES_METRIC_TS (SERIES_ID, METRIC_ID, TS),
  index IDX_SUMMARY_CALC (TS),
  constraint FK_LLRAW_SERIES_ID foreign key (SERIES_ID) references CMON_LONG_LIVED_TIME_SERIES_ID (ID)
) ENGINE=InnoDB;

create table CMON_ACTIVITY_TIME_SERIES_ID (
  ID         bigint not null auto_increment,
  NAME       varchar(255) not null,
  JOBID      varchar(255),
  primary key (ID),
  index IDX_ACTIVITY_SERIES_BY_NAME (NAME)
) ENGINE=InnoDB;

create table CMON_ACTIVITY_DATA_POINTS (
  ID          bigint not null auto_increment,
  SERIES_ID   bigint not null,
  METRIC_ID   bigint not null,
  TS          bigint not null,
  VALUE       double not null,
  primary key (ID),
  constraint FK_ACTRAW_SERIES_ID foreign key (SERIES_ID) references CMON_ACTIVITY_TIME_SERIES_ID (ID)
) ENGINE=InnoDB;
