-- Copyright (c) 2012 Cloudera, Inc. All rights reserved.
--
-- Version 28 of the CMF schema. Migration code to be applied on top of the
-- previous version. Technically, the schema isn't changed but we rely on
-- the schema version to trigger running of the ddl files.
--

update SCHEMA_VERSION set VERSION=28;

-- 1) Mutate TT final flag configs to TT overrides

update CONFIGS C1
    set C1.ATTR='override_mapred_child_java_opts_max_heap',
        C1.VALUE=(select C2.VALUE
                  from CONFIGS C2
                  where NVL(C1.ROLE_ID, 0)=NVL(C2.ROLE_ID, 0) and
                        NVL(C1.ROLE_TYPE, 'NULL')=NVL(C2.ROLE_TYPE, 'NULL') and
                        C1.SERVICE_ID=C2.SERVICE_ID and
                        C1.REVISION_ID=C2.REVISION_ID and
                        C2.ATTR='mapred_child_java_opts_max_heap')
    where dbms_lob.compare(C1.VALUE, 'true') = 0 and
          C1.ATTR='mapred_child_java_opts_max_heap_final' and
          exists (select C2.VALUE
                  from CONFIGS C2
                  where NVL(C1.ROLE_ID, 0)=NVL(C2.ROLE_ID, 0) and
                        NVL(C1.ROLE_TYPE, 'NULL')=NVL(C2.ROLE_TYPE, 'NULL') and
                        C1.SERVICE_ID=C2.SERVICE_ID and
                        C1.REVISION_ID=C2.REVISION_ID and
                        C2.ATTR='mapred_child_java_opts_max_heap');


update CONFIGS C1
    set C1.ATTR='override_mapred_child_ulimit',
        C1.VALUE=(select C2.VALUE
                  from CONFIGS C2
                  where NVL(C1.ROLE_ID, 0)=NVL(C2.ROLE_ID, 0) and
                        NVL(C1.ROLE_TYPE, 'NULL')=NVL(C2.ROLE_TYPE, 'NULL') and
                        C1.SERVICE_ID=C2.SERVICE_ID and
                        C1.REVISION_ID=C2.REVISION_ID and
                        C2.ATTR='mapred_child_ulimit')
    where dbms_lob.compare(C1.VALUE, 'true') = 0 and
          C1.ATTR='mapred_child_ulimit_final' and
          exists (select C2.VALUE
                  from CONFIGS C2
                  where NVL(C1.ROLE_ID, 0)=NVL(C2.ROLE_ID, 0) and
                        NVL(C1.ROLE_TYPE, 'NULL')=NVL(C2.ROLE_TYPE, 'NULL') and
                        C1.SERVICE_ID=C2.SERVICE_ID and
                        C1.REVISION_ID=C2.REVISION_ID and
                        C2.ATTR='mapred_child_ulimit');

update CONFIGS C1
    set C1.ATTR='override_mapred_job_reuse_jvm_num_tasks',
        C1.VALUE=(select C2.VALUE
                  from CONFIGS C2
                  where NVL(C1.ROLE_ID, 0)=NVL(C2.ROLE_ID, 0) and
                        NVL(C1.ROLE_TYPE, 'NULL')=NVL(C2.ROLE_TYPE, 'NULL') and
                        C1.SERVICE_ID=C2.SERVICE_ID and
                        C1.REVISION_ID=C2.REVISION_ID and
                        C2.ATTR='mapred_job_reuse_jvm_num_tasks')
    where dbms_lob.compare(C1.VALUE, 'true') = 0 and
          C1.ATTR='mapred_job_reuse_jvm_num_tasks_final' and
          exists (select C2.VALUE
                  from CONFIGS C2
                  where NVL(C1.ROLE_ID, 0)=NVL(C2.ROLE_ID, 0) and
                        NVL(C1.ROLE_TYPE, 'NULL')=NVL(C2.ROLE_TYPE, 'NULL') and
                        C1.SERVICE_ID=C2.SERVICE_ID and
                        C1.REVISION_ID=C2.REVISION_ID and
                        C2.ATTR='mapred_job_reuse_jvm_num_tasks');

-- 2) Move roleType level client configs from JOB/TASKTRACKER to BASTION

update CONFIGS
    set ROLE_TYPE='BASTION' where
        ROLE_TYPE='TASKTRACKER' and
        ATTR in ('mapred_output_compress',
                 'mapred_output_compression_type',
                 'mapred_output_compression_codec',
                 'mapred_compress_map_output',
                 'mapred_map_output_compression_codec',
                 'mapred_child_java_opts_base',
                 'mapred_child_java_opts_max_heap',
                 'mapred_child_ulimit',
                 'mapred_job_reuse_jvm_num_tasks',
                 'io_sort_factor',
                 'io_sort_mb',
                 'io_sort_record_percent',
                 'io_sort_spill_percent',
                 'mapred_reduce_parallel_copies');

update CONFIGS
    set ROLE_TYPE='BASTION' where
        ROLE_TYPE='JOBTRACKER' and
        ATTR='mapred_reduce_tasks';


-- 3) Rename TT instance values for autoconfigs

update CONFIGS
    set ATTR='autoconfig_mapred_child_java_opts_max_heap' where
        ROLE_ID is not null and
        ATTR='mapred_child_java_opts_max_heap';

update CONFIGS
    set ATTR='autoconfig_io_sort_mb' where
        ROLE_ID is not null and
        ATTR='io_sort_mb';

-- 4) Delete remaining TT configs

delete from CONFIGS where
    ROLE_ID is not null and
    ATTR in ('mapred_output_compress',
             'mapred_output_compression_type',
             'mapred_output_compression_codec',
             'mapred_compress_map_output',
             'mapred_map_output_compression_codec',
             'mapred_child_java_opts_base',
             'mapred_child_java_opts_max_heap',
             'mapred_child_ulimit',
             'mapred_job_reuse_jvm_num_tasks',
             'io_sort_factor',
             'io_sort_mb',
             'io_sort_record_percent',
             'io_sort_spill_percent',
             'mapred_reduce_parallel_copies',
             'mapred_reduce_tasks');

delete from CONFIGS where
    ATTR in ('mapred_child_java_opts_max_heap_final',
             'mapred_child_ulimit_final',
             'mapred_job_reuse_jvm_num_tasks_final');
