-- Copyright (c) 2011 Cloudera, Inc. All rights reserved.
--
-- PostgreSQL schema for CMF.

-- The schema version, should be checked by the server during startup
create table SCHEMA_VERSION (
    VERSION int4 not null
);

-- Update this value when the schema changes, also during migration
insert into SCHEMA_VERSION VALUES (3);

create table AUDITS (
    AUDIT_ID int8 not null,
    SERVICE_ID int8,
    ROLE_ID int8,
    CREATED_INSTANT bigint,
    MESSAGE varchar(1048576),
    primary key (AUDIT_ID)
);

create table COMMANDS (
    COMMAND_ID int8 not null,
    NAME varchar(255) not null,
    STATE varchar(255),
    START_INSTANT bigint,
    END_INSTANT bigint,
    ACTIVE int4,
    RESULT_MESSAGE varchar(1048576),
    RESULT_DATA bytea,
    RESULT_DATA_MIME_TYPE varchar(255),
    RESULT_DATA_FILENAME varchar(255),
    SUCCESS bool,
    SERVICE_ID int8,
    ROLE_ID int8,
    PARENT_ID int8,
    primary key (COMMAND_ID)
);

create table CONFIGS (
    CONFIG_ID int8 not null,
    ROLE_ID int8,
    ROLE_TYPE varchar(255),
    ATTR varchar(255) not null,
    VALUE varchar(1048576) not null,
    SERVICE_ID int8 not null,
    primary key (CONFIG_ID)
);

create table CREDENTIALS (
    CREDENTIAL_ID int8 not null,
    PRINCIPAL varchar(255) not null unique,
    KEYTAB bytea not null,
    primary key (CREDENTIAL_ID)
);

create table HOSTS (
    HOST_ID int8 not null,
    OPTIMISTIC_LOCK_VERSION int8 not null,
    HOST_IDENTIFIER varchar(255) not null unique,
    NAME varchar(255),
    IP_ADDRESS varchar(255),
    RACK_ID varchar(255),
    STATUS varchar(255),
    primary key (HOST_ID)
);

create table PROCESSES (
    PROCESS_ID int8 not null,
    NAME varchar(255) not null,
    ROLE_ID int8,
    COMMAND_ID int8,
    HOST_ID int8 not null,
    PROCESS_USER varchar(255),
    PROCESS_GROUP varchar(255),
    PROGRAM varchar(255),
    ARGUMENTS varchar(1048576),
    RUNNING bool,
    RUN_GENERATION int8,
    CONFIG_GENERATION int8 not null,
    ONE_OFF bool,
    CONFIGURATION_DATA bytea,
    STATUS_LINKS varchar(1048576),
    RESOURCES text,
    ENVIRONMENT text,
    REFRESH_FILES varchar(1048576),
    primary key (PROCESS_ID)
);

create table ROLES (
    ROLE_ID int8 not null,
    NAME varchar(255) not null unique,
    HOST_ID int8,
    ROLE_TYPE varchar(255),
    CONFIGURED_STATUS varchar(255),
    STATUS varchar(255),
    CONFIG_GENERATION int8 not null,
    SERVICE_ID int8,
    primary key (ROLE_ID)
);

create table SERVICES (
    SERVICE_ID int8 not null,
    OPTIMISTIC_LOCK_VERSION int8 not null,
    NAME varchar(255) not null unique,
    SERVICE_TYPE varchar(255),
    primary key (SERVICE_ID)
);

create table SETTINGS (
    SETTINGS_ID int8 not null,
    OPTIMISTIC_LOCK_VERSION int8 not null,
    HEARTBEAT_INTERVAL int8,
    MISSED_HB_CONCERNING int8,
    MISSED_HB_BAD int8,
    PASSWORD_HASH varchar(255),
    PASSWORD_SALT int8,
    WEB_TLS bool,
    AGENT_TLS bool,
    NEED_AGENT_VALIDAITON bool,
    KEYSTORE_PATH varchar(1048576),
    KEYSTORE_PASSWORD varchar(255),
    TRUSTSTORE_PATH varchar(1048576),
    TRUSTSTORE_PASSWORD varchar(255),
    HTTP_PORT int4,
    HTTPS_PORT int4,
    AGENT_PORT int4,
    SECURITY_REALM varchar(255),
    primary key (SETTINGS_ID)
);

create index IDX_COMMAND_PARENT on COMMANDS (PARENT_ID);

create index IDX_COMMAND_ROLE on COMMANDS (ROLE_ID);

create index IDX_COMMAND_SERVICE on COMMANDS (SERVICE_ID);

alter table COMMANDS 
    add constraint FK_COMMAND_ROLE 
    foreign key (ROLE_ID) 
    references ROLES;

alter table COMMANDS 
    add constraint FK_COMMAND_PARENT 
    foreign key (PARENT_ID) 
    references COMMANDS;

alter table COMMANDS 
    add constraint FK_COMMAND_SERVICE 
    foreign key (SERVICE_ID) 
    references SERVICES;

create index IDX_CONFIG_ROLE on CONFIGS (ROLE_ID);

create index IDX_CONFIG_SERVICE on CONFIGS (SERVICE_ID);

alter table CONFIGS 
    add constraint FK_CONFIG_ROLE 
    foreign key (ROLE_ID) 
    references ROLES;

alter table CONFIGS 
    add constraint FK_CONFIG_SERVICE 
    foreign key (SERVICE_ID) 
    references SERVICES;

create index IDX_PROCESS_ROLE on PROCESSES (ROLE_ID);

create index IDX_PROCESS_HOST on PROCESSES (HOST_ID);

create index IDX_PROCESS_COMMAND on PROCESSES (COMMAND_ID);

alter table PROCESSES 
    add constraint FK_PROCESS_HOST 
    foreign key (HOST_ID) 
    references HOSTS;

alter table PROCESSES 
    add constraint FK_PROCESS_ROLE 
    foreign key (ROLE_ID) 
    references ROLES;

alter table PROCESSES 
    add constraint FK_PROCESS_COMMAND 
    foreign key (COMMAND_ID) 
    references COMMANDS;

create index IDX_ROLE_SERVICE on ROLES (SERVICE_ID);

create index IDX_ROLE_HOST on ROLES (HOST_ID);

alter table ROLES 
    add constraint FK_ROLE_HOST 
    foreign key (HOST_ID) 
    references HOSTS;

alter table ROLES 
    add constraint FK_ROLE_SERVICE 
    foreign key (SERVICE_ID) 
    references SERVICES;
